@@ -2,7 +2,7 @@
 imgSeek ::  C++ database implementation
 ---------------------------------------
 begin                : Fri Jan 17 2003
-email                : nieder|at|mail.ru
+email                : ricardo.cabral|at|imgseek.net
 
 Copyright (C) 2003-2009 Ricardo Niederberger Cabral
 
@@ -51,14 +51,6 @@ using namespace std;
 #include "bloom_filter.h"
 #include "imgdb.h"
 
-#ifdef ISK_SWIG_JAVA
-/* Java interface */
-    #include "imgdb_wrap.h"
-#else
-    /* Python interface */
-    #include "imgdb_wrap.cxx"
-#endif
-
 //TODO reactivate fast jpeg loader: http://trac.xapian.org/browser/branches/imgseek/xapian-extras/imgseek
 
 // Globals
@@ -163,6 +155,11 @@ int addImageFromImage(const int dbId, const long int id, Image * image ) {
 	 */
 	if (!validate_dbid(dbId)) { cerr << "ERROR: database space not found (" << dbId << ")" << endl; return 0; }
 
+	if (dbSpace[dbId]->sigs.count(id)) {
+		cerr << "ERROR: imgId already in use" << endl;
+		return 0;
+	}
+
     if (image == (Image *) NULL) {
     	cerr << "ERROR: unable to add null image" << endl;
     	return 0;
@@ -214,7 +211,7 @@ int addImageFromImage(const int dbId, const long int id, Image * image ) {
 		pixel_cache++;
 	}
 
-    DestroyImage(resize_image);
+	DestroyImage(resize_image);
 
 	transformChar(rchan, gchan, bchan, cdata1, cdata2, cdata3);
 
@@ -225,14 +222,6 @@ int addImageFromImage(const int dbId, const long int id, Image * image ) {
 	nsig->width = width;
 	nsig->height = height;
 
-	if (dbSpace[dbId]->sigs.count(id)) {
-		delete dbSpace[dbId]->sigs[id];
-		dbSpace[dbId]->sigs.erase(id);
-
-		cerr << "ERROR: dbId already in use" << endl;
-		return 0;
-
-	}
 	// insert into sigmap
 	dbSpace[dbId]->sigs[id] = nsig;
 	// insert into ids bloom filter
@@ -287,10 +276,12 @@ int addImageFromImage(const int dbId, const long int id, Image * image ) {
 
 }
 
-int addImageBlob(const int dbId, const long int id, const void *blob, const long length) {
-	ExceptionInfo exception;
-	ImageInfo *image_info;
+int addImageBlob(const int dbId, const long int id, const char *blob, const long length) {
 
+	ExceptionInfo exception;
+	GetExceptionInfo(&exception);
+
+	ImageInfo *image_info;
 	image_info = CloneImageInfo((ImageInfo *) NULL);
 
 	Image *image = BlobToImage(image_info, blob, length, &exception);
@@ -516,7 +507,6 @@ int loadalldbs(char* filename) {
 		return 0;
 	}
 
-
 	for (int k = 0; k < sz; k++) { // for each db
 		f.read((char *) &(dbId), sizeof(int)); // db id
 		if (!f.good()) {
@@ -530,7 +520,6 @@ int loadalldbs(char* filename) {
 	return res;
 }
 
-
 int savedbtostream(const int dbId, std::ofstream& f) {
 	/*
 	Serialization order:
@@ -670,7 +659,7 @@ int savealldbs(char* filename) {
 	return res;
 }
 
-std::vector<double> queryImgDataFiltered(const int dbId, Idx * sig1, Idx * sig2, Idx * sig3, double *avgl, int numres, int sketch, bloom_filter* bfilter) {
+std::vector<double> queryImgDataFiltered(const int dbId, Idx * sig1, Idx * sig2, Idx * sig3, double *avgl, int numres, int sketch, bloom_filter* bfilter, bool colorOnly) {
 	int idx, c;
 	int pn;
 	Idx *sig[3] = { sig1, sig2, sig3 };
@@ -697,34 +686,34 @@ std::vector<double> queryImgDataFiltered(const int dbId, Idx * sig1, Idx * sig2,
 			}
 		}
 	}
-
-	for (int b = 0; b < NUM_COEFS; b++) {	// for every coef on a sig
-		for (c = 0; c < 3; c++) {
-			//TODO see if FAST_POW_GEERT gives the same results
+    if (!colorOnly) {
+        for (int b = 0; b < NUM_COEFS; b++) {	// for every coef on a sig
+            for (c = 0; c < 3; c++) {
+                //TODO see if FAST_POW_GEERT gives the same results
 #ifdef FAST_POW_GEERT
-			pn  = sig[c][b] < 0;
-			idx = (sig[c][b] - pn) ^ -pn;
+                pn  = sig[c][b] < 0;
+                idx = (sig[c][b] - pn) ^ -pn;
 #else
-			pn = 0;
-			if (sig[c][b]>0) {
-				pn = 0;
-				idx = sig[c][b];
-			} else {
-				pn = 1;
-				idx = -sig[c][b];
-			}
+                pn = 0;
+                if (sig[c][b]>0) {
+                    pn = 0;
+                    idx = sig[c][b];
+                } else {
+                    pn = 1;
+                    idx = -sig[c][b];
+                }
 #endif
 
-
-			// update the score of every image which has this coef
-			long_listIterator end = dbSpace[dbId]->imgbuckets[c][pn][idx].end();
-			for (long_listIterator uit = dbSpace[dbId]->imgbuckets[c][pn][idx].begin();
-			uit != end; uit++) {
-				//TODO in each iteration search in tree (std::map) is performed. i think the better way to link by pointers.
-				dbSpace[dbId]->sigs[(*uit)]->score -= weights[sketch][imgBin[idx]][c];
-			}
-		}
-	}
+                // update the score of every image which has this coef
+                long_listIterator end = dbSpace[dbId]->imgbuckets[c][pn][idx].end();
+                for (long_listIterator uit = dbSpace[dbId]->imgbuckets[c][pn][idx].begin();
+                uit != end; uit++) {
+                    //TODO in each iteration search in tree (std::map) is performed. i think the better way to link by pointers.
+                    dbSpace[dbId]->sigs[(*uit)]->score -= weights[sketch][imgBin[idx]][c];
+                }
+            }
+        }
+    }
 
 	sigPriorityQueue pqResults;		/* results priority queue; largest at top */
 
@@ -732,22 +721,15 @@ std::vector<double> queryImgDataFiltered(const int dbId, Idx * sig1, Idx * sig2,
 
 	vector<double> V;
 
+	pqResults.push(*(*sit).second);
+	sit++;
+
 	// Fill up the numres-bounded priority queue (largest at top):
-	for (int cnt = 0; cnt < numres; cnt++) {
-		if (sit == dbSpace[dbId]->sigs.end()) {
-			// No more images; cannot get requested numres, so just return these initial ones.
-			return V;
-		}
-		pqResults.push(*(*sit).second);
-		sit++;
-	}
-
-
 	for (; sit != dbSpace[dbId]->sigs.end(); sit++) {
 		// only consider if not ignored due to keywords and if is a better match than the current worst match
-		if (((*sit).second->score < 99999) && ((*sit).second->score < pqResults.top().score)) {
+		if ((*sit).second->score < 99999) {
 			// Make room by dropping largest entry:
-			pqResults.pop();
+			if (pqResults.size() >= numres) pqResults.pop();
 			// Insert new entry:
 			pqResults.push(*(*sit).second);
 		}
@@ -764,7 +746,6 @@ std::vector<double> queryImgDataFiltered(const int dbId, Idx * sig1, Idx * sig2,
 	}
 
 	return V;
-
 }
 
 
@@ -773,63 +754,8 @@ avgl is the average luminance
 numres is the max number of results
 sketch (0 or 1) tells which set of weights to use
  */
-std::vector<double> queryImgData(const int dbId, Idx * sig1, Idx * sig2, Idx * sig3, double *avgl, int numres, int sketch) {
-	return queryImgDataFiltered(dbId, sig1, sig2, sig3, avgl, numres, sketch, 0);
-
-}
-
-/* Will only look at avg lum
- * sig1,2,3 are int arrays of length NUM_COEFS
-avgl is the average luminance
-numres is the max number of results
-sketch (0 or 1) tells which set of weights to use
- */
-std::vector<double> queryImgDataFast(const int dbId, Idx * sig1, Idx * sig2, Idx * sig3, double *avgl, int numres, int sketch) {
-	int c;
-
-	vector<double> V;
-
-	if (!validate_dbid(dbId)) { cerr << "ERROR: database space not found (" << dbId << ")" << endl; return V;}
-
-	for (sigIterator sit = dbSpace[dbId]->sigs.begin(); sit != dbSpace[dbId]->sigs.end(); sit++) {
-		(*sit).second->score = 0;
-		for (c = 0; c < 3; c++) {
-			(*sit).second->score += weights[sketch][0][c]
-			                                           * fabs((*sit).second->avgl[c] - avgl[c]);
-		}
-	}
-
-	sigPriorityQueue pqResults;		/* results priority queue; largest at top */
-	sigIterator sit = dbSpace[dbId]->sigs.begin();
-
-
-	// Fill up the numres-bounded priority queue (largest at top):
-	for (int cnt = 0; cnt < numres; cnt++) {
-		if (sit == dbSpace[dbId]->sigs.end())
-			// No more images; cannot get requested numres, alas.
-			return V;
-		pqResults.push(*(*sit).second);
-		sit++;
-	}
-
-	for (; sit != dbSpace[dbId]->sigs.end(); sit++) {
-		if ((*sit).second->score < pqResults.top().score) {
-			// Make room by dropping largest entry:
-			pqResults.pop();
-			// Insert new entry:
-			pqResults.push(*(*sit).second);
-		}
-	}
-
-	SigStruct curResTmp;            /* current result waiting to be returned */
-	while (pqResults.size()) {
-		curResTmp = pqResults.top();
-		pqResults.pop();
-		V.insert(V.end(), curResTmp.id);
-		V.insert(V.end(), curResTmp.score);
-	}
-
-	return V;
+std::vector<double> queryImgData(const int dbId, Idx * sig1, Idx * sig2, Idx * sig3, double *avgl, int numres, int sketch, bool colorOnly) {
+	return queryImgDataFiltered(dbId, sig1, sig2, sig3, avgl, numres, sketch, 0, colorOnly);
 
 }
 
@@ -910,49 +836,150 @@ long_list queryImgDataForThresFast(sigMap * tsigs, double *avgl, float thresd, i
 	return res;
 }
 
-// cluster by similarity. Returns list of list of long ints (img ids)
-/*
-long_list_2 clusterSim(const int dbId, float thresd, int fast = 0) {
-	long_list_2 res;		// will hold a list of lists. ie. a list of clusters
-	sigMap wSigs(dbSpace[dbId]->sigs);		// temporary map of sigs, as soon as an image becomes part of a cluster, it's removed from this map
-	sigMap wSigsTrack(dbSpace[dbId]->sigs);	// temporary map of sigs, as soon as an image becomes part of a cluster, it's removed from this map
+//TODO some places are using double_vector others std::vector. Decide!
+std::vector<double>  queryImgBlob(const int dbId, const char* data,const long length, int numres,int sketch, bool colorOnly) {
+	ExceptionInfo exception;
+	GetExceptionInfo(&exception);
 
-	for (sigIterator sit = wSigs.begin(); sit != wSigs.end(); sit++) {	// for every img on db
-		long_list res2;
+	ImageInfo *image_info;
+	image_info = CloneImageInfo((ImageInfo *) NULL);
 
-		if (fast) {
-			res2 =
-				queryImgDataForThresFast(&wSigs, (*sit).second->avgl,
-				thresd, 1);
-		} else {
-			res2 =
-				queryImgDataForThres(dbId, &wSigs, (*sit).second->sig1,
-				(*sit).second->sig2,
-				(*sit).second->sig3,
-				(*sit).second->avgl, thresd, 1);
-		}
-		//    continue;
-		long int hid = (*sit).second->id;
-		//    if ()
-		wSigs.erase(hid);
-		if (res2.size() <= 1) {
-			if (wSigs.size() <= 1)
-				break;		// everything already added to a cluster sim. Bail out immediately, otherwise next iteration
-			// will segfault when incrementing sit
-			continue;
-		}
-		res2.push_front(hid);
-		res.push_back(res2);
-		if (wSigs.size() <= 1)
-			break;
-		// sigIterator sit2 = wSigs.end();
-		//    sigIterator sit3 = sit++;
+	Image *image = BlobToImage(image_info, data, length, &exception);
+	if (exception.severity != UndefinedException) CatchException(&exception);
+
+	DestroyImageInfo(image_info);
+
+	// Made static for speed; only used locally
+	static Unit cdata1[16384];
+	static Unit cdata2[16384];
+	static Unit cdata3[16384];
+	int i;
+
+	Image *resize_image;
+
+	/*
+	Initialize the image info structure and read an image.
+	 */
+	GetExceptionInfo(&exception);
+
+	resize_image = SampleImage(image, 128, 128, &exception);
+
+	DestroyImage(image);
+
+	DestroyExceptionInfo(&exception);
+
+	if (resize_image == (Image *) NULL) {
+		cerr << "ERROR: unable to resize image" << endl;
+    	return vector<double>();
 	}
-	return res;
-}
- */
 
-std::vector<double> queryImgID(const int dbId, long int id, int numres) {
+    // store color value for basic channels
+	unsigned char rchan[16384];
+	unsigned char gchan[16384];
+	unsigned char bchan[16384];
+
+	GetExceptionInfo(&exception);
+
+	const PixelPacket *pixel_cache = AcquireImagePixels(resize_image, 0, 0, 128, 128, &exception);
+
+	for (int idx = 0; idx < 16384; idx++) {
+		rchan[idx] = pixel_cache->red;
+		gchan[idx] = pixel_cache->green;
+		bchan[idx] = pixel_cache->blue;
+		pixel_cache++;
+	}
+
+    DestroyImage(resize_image);
+
+	transformChar(rchan, gchan, bchan, cdata1, cdata2, cdata3);
+
+	DestroyExceptionInfo(&exception);
+
+	SigStruct *nsig = new SigStruct();
+    //TODO leaking nsig?
+	calcHaar(cdata1, cdata2, cdata3,
+			nsig->sig1, nsig->sig2, nsig->sig3, nsig->avgl);
+
+	return queryImgData(dbId, nsig->sig1, nsig->sig2, nsig->sig3,
+			nsig->avgl, numres, sketch, colorOnly);
+}
+
+std::vector<double> queryImgPath(const int dbId, char* path,int numres,int sketch, bool colorOnly) {
+
+	ExceptionInfo exception;
+	GetExceptionInfo(&exception);
+
+	ImageInfo *image_info;
+	image_info = CloneImageInfo((ImageInfo *) NULL);
+	(void) strcpy(image_info->filename, path);
+	Image *image = ReadImage(image_info, &exception);
+	if (exception.severity != UndefinedException) CatchException(&exception);
+	DestroyImageInfo(image_info);
+	DestroyExceptionInfo(&exception);
+
+	if (image == (Image *) NULL) {
+    	cerr << "ERROR: unable to read image" << endl;
+    	return vector<double>();
+    }
+
+	// Made static for speed; only used locally
+	static Unit cdata1[16384];
+	static Unit cdata2[16384];
+	static Unit cdata3[16384];
+	int i;
+
+	Image *resize_image;
+
+	/*
+	Initialize the image info structure and read an image.
+	 */
+	GetExceptionInfo(&exception);
+
+	resize_image = SampleImage(image, 128, 128, &exception);
+
+	DestroyImage(image);
+
+	DestroyExceptionInfo(&exception);
+
+	if (resize_image == (Image *) NULL) {
+		cerr << "ERROR: unable to resize image" << endl;
+    	return vector<double>();
+	}
+
+    // store color value for basic channels
+	unsigned char rchan[16384];
+	unsigned char gchan[16384];
+	unsigned char bchan[16384];
+
+	GetExceptionInfo(&exception);
+
+	const PixelPacket *pixel_cache = AcquireImagePixels(resize_image, 0, 0, 128, 128, &exception);
+
+	for (int idx = 0; idx < 16384; idx++) {
+		rchan[idx] = pixel_cache->red;
+		gchan[idx] = pixel_cache->green;
+		bchan[idx] = pixel_cache->blue;
+		pixel_cache++;
+	}
+
+    DestroyImage(resize_image);
+
+	transformChar(rchan, gchan, bchan, cdata1, cdata2, cdata3);
+
+	DestroyExceptionInfo(&exception);
+
+	SigStruct *nsig = new SigStruct();
+
+	calcHaar(cdata1, cdata2, cdata3,
+			nsig->sig1, nsig->sig2, nsig->sig3, nsig->avgl);
+
+	return queryImgData(dbId, nsig->sig1, nsig->sig2, nsig->sig3,
+			nsig->avgl, numres, sketch, colorOnly);
+}
+
+
+//TODO add parm for query tweaking (sketch?)
+std::vector<double> queryImgID(const int dbId, long int id, int numres, int sketch, bool colorOnly) {
 	/*query for images similar to the one that has this id
 	numres is the maximum number of results
 	 */
@@ -986,27 +1013,17 @@ std::vector<double> queryImgID(const int dbId, long int id, int numres) {
 	if (!validate_imgid(dbId, id)) { cerr << "ERROR: image id (" << id << ") not found on given dbid (" << dbId << ") or dbid not existant" << endl ; return std::vector<double>();};
 
 	return queryImgData(dbId, dbSpace[dbId]->sigs[id]->sig1, dbSpace[dbId]->sigs[id]->sig2, dbSpace[dbId]->sigs[id]->sig3,
-			dbSpace[dbId]->sigs[id]->avgl, numres, 0);
+			dbSpace[dbId]->sigs[id]->avgl, numres, sketch, colorOnly);
 }
 
-std::vector<double> queryImgIDFiltered(const int dbId, long int id, int numres, bloom_filter* bf) {
+std::vector<double> queryImgIDFiltered(const int dbId, long int id, int numres, bloom_filter* bf, bool colorOnly) {
 	/*query for images similar to the one that has this id
 	numres is the maximum number of results
 	 */
 
 	if (!validate_imgid(dbId, id)) { cerr << "ERROR: image id (" << id << ") not found on given dbid (" << dbId << ") or dbid not existant" << endl ; return std::vector<double>();};
 	return queryImgDataFiltered(dbId, dbSpace[dbId]->sigs[id]->sig1, dbSpace[dbId]->sigs[id]->sig2, dbSpace[dbId]->sigs[id]->sig3,
-			dbSpace[dbId]->sigs[id]->avgl, numres, 0, bf);
-}
-
-std::vector<double> queryImgIDFast(const int dbId, long int id, int numres) {
-	/*query for images similar to the one that has this id
-	numres is the maximum number of results
-	 */
-	if (!validate_imgid(dbId, id)) { cerr << "ERROR: image id (" << id << ") not found on given dbid (" << dbId << ") or dbid not existant" << endl ; return std::vector<double>();};
-
-	return queryImgDataFast(dbId, dbSpace[dbId]->sigs[id]->sig1, dbSpace[dbId]->sigs[id]->sig2, dbSpace[dbId]->sigs[id]->sig3,
-			dbSpace[dbId]->sigs[id]->avgl, numres, 0);
+			dbSpace[dbId]->sigs[id]->avgl, numres, 0, bf, colorOnly);
 }
 
 int removeID(const int dbId, long int id) {
@@ -1075,28 +1092,18 @@ int destroydb(const int dbId) {
 int resetdb(const int dbId) {
 
 	if (!validate_dbid(dbId)) { cerr << "ERROR: database space not found (" << dbId << ")" << endl; return 0;}
-	//TODO delete kwdstructs from globalKwdsMap
-	//TODO there is a memleak here:
-    // ==19336==    at 0x4C25F6C: operator new(unsigned long) (vg_replace_malloc.c:230)
-    // ==19336==    by 0x878CFE0: std::string::_Rep::_S_create(unsigned long, unsigned long, std::allocator<char> const&) (in /usr/lib/libstdc++.so.6.0.10)
-    // ==19336==    by 0x878D88A: std::string::_Rep::_M_clone(std::allocator<char> const&, unsigned long) (in /usr/lib/libstdc++.so.6.0.10)
-    // ==19336==    by 0x878E2AB: std::string::string(std::string const&) (in /usr/lib/libstdc++.so.6.0.10)
-    // ==19336==    by 0x78B046F: std::vector<std::string, std::allocator<std::string> >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > >, std::string const&) (new_allocator.h:108)
-    // ==19336==    by 0x78B4F40: bloom_filter::generate_unique_salt() (stl_vector.h:694)
-    // ==19336==    by 0x78968D8: resetdb(int) (bloom_filter.h:58)
-    // ==19336==    by 0x78982FC: _wrap_resetdb (imgdb_python_linux_wrap.h:11785)
-
     // first deallocate db memory
 
 	// deallocate all buckets
 
     for (int c = 0; c < 3; c++)
         for (int pn = 0; pn < 2; pn++)
-            for (int i = 0; i < 16384; i++)
-                dbSpace[dbId]->imgbuckets[c][pn][i].clear();
+            for (int i = 0; i < 16384; i++) {
+                if (dbSpace[dbId])
+                    dbSpace[dbId]->imgbuckets[c][pn][i].clear();
+            }
 
     //delete sigs
-
     for (sigIterator it = dbSpace[dbId]->sigs.begin(); it != dbSpace[dbId]->sigs.end(); it++) {
         delete (*it).second;
     }
@@ -1106,14 +1113,13 @@ int resetdb(const int dbId) {
     dbSpace[dbId]->sigs.clear(); // this is making windows choke
     // dbSpace[dbId]->sigs = sigMap();
 
-	delete dbSpace[dbId];
- 	dbSpace.erase(dbId);
+	//delete dbSpace[dbId];
+ 	//dbSpace.erase(dbId);
 
  	// finally the reset itself
 	dbSpace[dbId] = new dbSpaceStruct();
 
 	return 1;
-
 }
 
 long int getImgCount(const int dbId) {
@@ -1219,7 +1225,6 @@ std::vector<int> getKeywordsImg(const int dbId, const int id){
 	return ret;
 }
 
-// query by keywords
 std::vector<int> mostPopularKeywords(const int dbId, std::vector<long int> imgs, std::vector<int> excludedKwds, int count, int mode) {
 
 	kwdFreqMap freqMap = kwdFreqMap();
@@ -1262,11 +1267,10 @@ std::vector<int> mostPopularKeywords(const int dbId, std::vector<long int> imgs,
 	}
 
 	return res;
-
 }
 
 // query by keywords
-std::vector<double> queryImgIDKeywords(const int dbId, long int id, int numres, int kwJoinType, int_vector keywords){
+std::vector<double> queryImgIDKeywords(const int dbId, long int id, int numres, int kwJoinType, int_vector keywords, bool colorOnly){
 	if (!validate_dbid(dbId)) { cerr << "ERROR: database space not found (" << dbId << ")" << endl; return std::vector<double>();}
 
 	if ((id != 0) && !validate_imgid(dbId, id)) { // not search random and image doesnt exist
@@ -1274,24 +1278,26 @@ std::vector<double> queryImgIDKeywords(const int dbId, long int id, int numres,
 		return std::vector<double>();
 	}
 
+	if (keywords.size() < 1) { 
+		cerr << "ERROR: At least one keyword must be supplied" << endl ;
+		return std::vector<double>();        
+	} 
+
 	// populate filter
 	intVectorIterator it = keywords.begin();
 	bloom_filter* bf = 0;
-	if (*it == 0) {
-		// querying all tags
-	} else {
-		// OR or AND each kwd postings filter to get final filter
-		// start with the first one
-		bf = new bloom_filter(*(getKwdPostings(*it)->imgIdsFilter));
-		it++;
-		for (; it != keywords.end(); it++) { // iterate the rest
-			if (kwJoinType) { // and'd
-				(*bf) &= *(getKwdPostings(*it)->imgIdsFilter);
-			} else { // or'd
-				(*bf) |= *(getKwdPostings(*it)->imgIdsFilter);
-			}
-		}
-	}
+
+    // OR or AND each kwd postings filter to get final filter
+    // start with the first one
+    bf = new bloom_filter(*(getKwdPostings(*it)->imgIdsFilter));
+    it++;
+    for (; it != keywords.end(); it++) { // iterate the rest
+        if (kwJoinType) { // and'd
+            (*bf) &= *(getKwdPostings(*it)->imgIdsFilter);
+        } else { // or'd
+            (*bf) |= *(getKwdPostings(*it)->imgIdsFilter);
+        }
+    }
 
 	if (id == 0) { // random images with these kwds
 
@@ -1320,19 +1326,10 @@ std::vector<double> queryImgIDKeywords(const int dbId, long int id, int numres,
 
 		return Vres;
 	}
-	return queryImgIDFiltered(dbId, id, numres, bf);
+	return queryImgIDFiltered(dbId, id, numres, bf, colorOnly);
 
 }
-std::vector<double> queryImgIDFastKeywords(const int dbId, long int id, int numres, int kwJoinType, int_vector keywords){
-	if (!validate_imgid(dbId, id)) { cerr << "ERROR: image id (" << id << ") not found on given dbid (" << dbId << ") or dbid not existant" << endl ; return std::vector<double>();};
 
-	throw string("not yet implemented");
-}
-std::vector<double> queryImgDataFastKeywords(const int dbId, int * sig1, int * sig2, int * sig3, double *avgl, int numres, int sketch, int kwJoinType, std::vector<int> keywords){
-	if (!validate_dbid(dbId)) { cerr << "ERROR: database space not found (" << dbId << ")" << endl; return std::vector<double>();}
-
-	throw string("not yet implemented");
-}
 std::vector<long int> getAllImgsByKeywords(const int dbId, const int numres, int kwJoinType, std::vector<int> keywords){
 	if (!validate_dbid(dbId)) { cerr << "ERROR: database space not found (" << dbId << ")" << endl; return std::vector<long int>();}
 
